/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* NOTE: printprimdata and printpriminfo could be merged, dont'
 * know if it will speed up the thing, maybe if they get inlined
 * they are faster separated (no "what I should print" guessing)
 * but if they are merged and they are inlined then the file will
 * be both bigger and with 2 extra guessing. Haaa undestanding
 * today compiler optimizations is hard
 */

#include "BDFile.h"
#include "u2d_core.h"

BDFile::BDFile() {
  bfilename[0] = '\0';
  dfilename[0] = '\0';
  firstprimitive = NULL;
  lastprimitive = NULL;
  elements = 0;
  vertexs = 0; /* discover what is this */
}


void BDFile::select_prim_list(Primitive *first, Primitive *last, int prim_number, int vertex_n){
  firstprimitive = first;
  lastprimitive = last;
  elements = prim_number;
  vertexs = vertex_n;
}
int BDFile::openbfile() {
  bfile = fopen(bfilename, "w+");
  if(bfile == NULL) {
    fprintf(stderr, "Can't open boundary file\n");
    return -1;
  }
  return 0;
}

int BDFile::opendfile() {
  dfile = fopen(dfilename, "w+");
  if(dfile == NULL) {
    fprintf(stderr, "Can't open domain file\n");
    return -1;
  }
  return 0;
}

/* FILE GENERATION */

inline void BDFile::printhash(FILE *output) {
  fprintf(output, "#\n");
}

inline void BDFile::printgeom(FILE *output) {
  fprintf(output, "#\n" \
                  "######## GEOMETRY###########\n" \
	          "#\n"\
	  );
}

inline void BDFile::printtopo(FILE *output) {
  fprintf(output, "#\n" \
                  "######## TOPOLOGY AND METRIC#######\n" \
	          "#\n"\
	  );
}

inline void BDFile::printnvne(FILE *output) {
  fprintf(output, "nv      ne\n");
}

void BDFile::printverelem(FILE *output) {
  if(vertexs > 9) fprintf(output, "%i", vertexs);
  else fprintf(output, " %i", vertexs);
  if(elements > 9) fprintf(output, "      %i\n", elements);
  else fprintf(output, "      %i\n", elements);
}

inline void BDFile::printish(FILE *output) {
  /*  fprintf(output, "   idata   sdata   hdata\n"); */
  fprintf(output, "       i       s       h\n");
}

inline void BDFile::printenbvev(FILE *output) {
  fprintf(output, "       e       n      bv      ev\n");
}

void BDFile::printprimg(FILE *output, Primitive *curprim) {
    if(strlen(curprim->primname) > 0){
      printbegin(output, curprim->element, curprim->primname);
    }
    else{
      printbegin(output, curprim->element);
    }
  switch(curprim->type) {
    case PR_LINE:
      fprintf(output, "line\n");
      fprintf(output, "%i %.2f %.2f %.2f %.2f\n", ((Line *)curprim)->spline, ((Line *)curprim)->x1, ((Line *)curprim)->y1, ((Line *)curprim)->x2, ((Line *)curprim)->y2);
      printend(output);
      break;
    case PR_CIRCLE:
      fprintf(output, "circle\n");
      fprintf(output, "%i %.2f %.2f %.2f %.2f %.2f\n", ((Circle *)curprim)->spline, ((Circle *)curprim)->centerx, ((Circle *)curprim)->centery, ((Circle *)curprim)->radius, ((Circle *)curprim)->startarc, ((Circle *)curprim)->endarc);
      printend(output);
      break;
    case PR_ELLIPSE:
      fprintf(output, "ellipse\n");
      fprintf(output, "%i %.2f %.2f %.2f %.2f %.2f %.2f\n", ((Ellipse *)curprim)->spline, ((Ellipse *)curprim)->centerx, ((Ellipse *)curprim)->centery, ((Ellipse *)curprim)->semix, ((Ellipse *)curprim)->semiy, ((Ellipse *)curprim)->startarc, ((Ellipse *)curprim)->endarc);
      printend(output);
      break;
    case PR_DATA:
      fprintf(output, "data\n");
      fprintf(output, "%s\n", ((Data *)curprim)->filename);
      fprintf(output, "   translation\n");
      fprintf(output, "   %.2f   %.2f\n", ((Data *)curprim)->trasx,\
	      ((Data *)curprim)->trasy);
      fprintf(output, "   rotation\n");
      fprintf(output, "   %.2f   %.2f\n", ((Data *)curprim)->rotation);
      printend(output);
      break;
    default:
      fprintf(stderr, "PRINTING: Unknown primitive type\n");
    }
}

void BDFile::printprimd(FILE *output, Primitive *curprim) {
  printbegin(output, curprim->element);
  printenbvev(output);
  fprintf(output, "       %i       %i       %i       %i\n", \
	  curprim->element, curprim->curdata, \
	  curprim->beginvertex, curprim->endvertex);
  printish(output);
  if(curprim->data != NULL) {
    for(int j = 0; j < curprim->curdata; j++) {
      fprintf(output, "       %.0f    %.2f    %.2f\n", curprim->getdata(j, 0), curprim->getdata(j, 1), curprim->getdata(j, 2));
    }
  }
  printend(output);
}

void BDFile::printbegin(FILE *output, int id) {
  fprintf(output, "BEGIN %i\n", id);
}

void BDFile::printbegin(FILE *output, int id, char *name) {
  fprintf(output, "BEGIN %i (%s)\n", id, name);
}

inline void BDFile::printend(FILE *output) {
  fprintf(output, "END\n");
}

/* add firstprimitive as an argument to printptiminfo and printprimdata */
void BDFile::printpriminfo(FILE *output) {
  Primitive *currentprimitive;
  currentprimitive = firstprimitive;
  for(int i = 1; i <= elements; i++) {
    printprimg(output, currentprimitive);
    if(currentprimitive->nextprimitive != NULL) {
      printhash(output);
      currentprimitive = currentprimitive->nextprimitive;
    }
    else if (currentprimitive->nextprimitive == NULL && i != elements) {
      fprintf(stderr, "printpriminfo: WTF!? How we are here?, elements say we still have" \
                      "primitive to print but currentprimitive->nextprimitive" \
                      "pointer is NULL\n");
    }
  }
}

void BDFile::printprimdata(FILE *output) {
  Primitive *currentprimitive;
  currentprimitive = firstprimitive;
  for(int i = 1; i <= elements; i++) {
    printprimd(output, currentprimitive);
    if(currentprimitive->nextprimitive != NULL) {
      printhash(output);
      currentprimitive = currentprimitive->nextprimitive;
    }
    else if (currentprimitive->nextprimitive == NULL && i != elements) {
      fprintf(stderr, "printprimdata: WTF!? How we are here?, elements say we still" \
                      "have primitive to print but currentprimitive->nextprimitive" \
                      "pointer is NULL\n");
    }
  }
}

/* FILE OUTPUT */

void BDFile::addboundary(char *buffer) {
  char *temp;
  char *filename;
  //  char *path;
  int pathlen;
  int bufferlen;
  int filelen;
  bufferlen = strlen(buffer);
  if(bufferlen >= 246) {  //boundary. is 9 letters
    fprintf(stderr, "Filename too long: it will be truncated\n");
    bufferlen = 246;
  }
  temp = rindex(buffer, '/');   // add a define for portability windows use the wrong slash '\'
  if(temp != NULL) {
    filename = temp +1;
  }
  else {
    filename = buffer;
  }
    //    path = buffer;
  filelen = strlen(filename);
  temp = (char *)malloc(filelen + 1);
  pathlen = bufferlen - filelen;
  strcpy(temp, filename);
  strncpy(filename, "boundary.", 9);
  strncpy(filename + 9, temp, 245 - pathlen); //245 need space for \0
  //  filename[filelen + 10] = '\0'; look unuseful, look like allready terminated correctly
  fprintf(stdout, "Filename modified to: %s\n", buffer);
  free(temp);
}

void BDFile::adddomain(char *buffer) {
  char *temp;
  char *filename;
  //  char *path;
  int pathlen;
  int bufferlen;
  int filelen;
  bufferlen = strlen(buffer);
  if(bufferlen >= 246) {  //boundary. is 9 letters
    fprintf(stderr, "Filename too long: it will be truncated\n");
    bufferlen = 246;
  }
  temp = rindex(buffer, '/');   // add a define for portability windows use the wrong slash '\'
  if(temp != NULL) {
    filename = temp +1;
  }
  else {
    filename = buffer;
  }
    //    path = buffer;
  filelen = strlen(filename);
  temp = (char *)malloc(filelen + 1);
  pathlen = bufferlen - filelen;
  strcpy(temp, filename);
  strncpy(filename, "domain.", 7);
  strncpy(filename + 7, temp, 245 - pathlen); //245 need space for \0
  //  filename[filelen + 10] = '\0'; look unuseful, look like allready terminated correctly
  fprintf(stdout, "Filename modified to: %s\n", buffer);
  free(temp);
}

/* for now output file will take firstprimitive as an argument, add it */
int BDFile::outputbfile() {
  /* Check if the list has been initialized correctly with
   * select_prim_list() */
  if(firstprimitive == NULL || elements == 0 || lastprimitive == NULL){
    fprintf(stdout, "Primitive list is has not been initialized, propely"\
	    "or is empty.\n");
    return -3; // notting to output or initialization error
  }
  /* Check for a valid filename */
  if(bfilename[0] == '\0') {
    fprintf(stderr, "outputbfile: Invalid filename\n");
    return -1; // invalid filename
  }
  if(strstr(bfilename, "boundary.") == NULL) {
    fprintf(stdout, "outputbfile: filename does not contain \"boundary.\", adding it\n");
    addboundary(bfilename);
  }
  /* Open the file */
  if(openbfile() != 0) {
    fprintf(stderr, "error opening the boundary file\n");
    return -2;  // file opening failed
  }
  /* Print the file */
  printnvne(bfile);
  printverelem(bfile);
  printgeom(bfile);
  printpriminfo(bfile);
  printtopo(bfile);
  printprimdata(bfile);
  fclose(bfile);
  return 0;
}

int BDFile::outputdfile() {

  /* Check for a valid filename */
  if(dfilename[0] == '\0') {
    fprintf(stderr, "outputdfile: Invalid filename\n");
    return -1; // invalid filename
  }
  if(strstr(bfilename, "domain.") == NULL) {
    fprintf(stdout, "outputdfile: filename does not contain \"domain.\", adding it\n");
    adddomain(dfilename);
  }
  /* Open the file */
  if(opendfile() != 0) {
    fprintf(stderr, "error opening the domain file\n");
    return -2;  // file opening failed
  }

  fprintf(dfile,\
	  "    NSTEP    LSMOOTH(1/0)\n"\
	  "    1000    1\n");

  printhash(dfile);

  fprintf(dfile,\
	  "    N	(BACKGRID POINTS)\n"\
	  "   0\n"\
	  "     X        Y	   HDATA\n");

  printhash(dfile);

  fprintf(dfile,\
	  "   N	(STEINER POINTS)\n"\
	  "   0\n"\
	  "     X        Y	    \n");
  fclose(dfile);
  return 0;
}
